   /**
  * @author Derek Marley derek_marley@kaltire.com
  * @version 1.0
  * Service: FileCopyService
  * Date   : Jan 17, 2008
  * Purpose: This service is used to copy file(s) on the file system 
  *  
  * Date     Initials  Description
  * -------- --------  ----------------------------------------------
  * 01.27.08 Dm        Created
  */

package com.progress.codeshare.esbservice.fileCopy;

import java.util.Iterator;
 
import com.sonicsw.xq.*;

public class FileCopyService implements XQService {

	private String srcFile          = null;
	private String destDir          = null;
	private String destFile         = null;
	private boolean removeSrc       = false;
    private boolean replaceExisting = false;
    private boolean createDir       = true;
    
	// This is the XQLog (the container's logging mechanism).
    private XQLog m_xqLog = null;

    /**
     * Constructor for a FileCopyService
     */
	public FileCopyService () {	
	}
	
	/**
     * Initialize the XQService by processing its initialization parameters.
     *
     * <p> This method implements a required XQService method.
     *
     * @param initialContext The Initial Service Context provides access to:<br>
     * <ul>
     * <li>The configuration parameters for this instance of the FileCopyService.</li>
     * <li>The XQLog for this instance of the FileCopyService.</li>
     * </ul>
     * @exception XQServiceException Used in the event of some error.
     */
    public void init(XQInitContext initialContext) throws XQServiceException {
       // XQParameters params = initialContext.getParameters();
        m_xqLog = initialContext.getLog();

        m_xqLog.logInformation("[FileCopyService] Initializing ...");
    }


    /**
     * Handle the arrival of XQMessages in the INBOX.
     *
     * <p> This method implement a required XQService method.
     *
     * @param ctx The service context.
     * @exception XQServiceException Thrown in the event of a processing error.
     */
    public void service(XQServiceContext ctx) throws XQServiceException {
        if (ctx == null)
            throw new XQServiceException("Service Context cannot be null.");
        else {

            // Get the message.
            XQEnvelope env = null;
            while (ctx.hasNextIncoming()) {
                env = ctx.getNextIncoming();
                XQParameters params = ctx.getParameters();
                srcFile    = params.getParameter("srcFile",  XQConstants.PARAM_STRING);
                destDir    = params.getParameter("destDir",  XQConstants.PARAM_STRING);
                destFile   = params.getParameter("destFile", XQConstants.PARAM_STRING);
                
                removeSrc       = params.getBooleanParameter("removeSrc", XQConstants.PARAM_STRING);
                replaceExisting = params.getBooleanParameter("replaceExisting", XQConstants.PARAM_STRING);
                createDir       = params.getBooleanParameter("createDir", XQConstants.PARAM_STRING);
                
        		m_xqLog.logInformation("echo srcFile:" + srcFile);
        		m_xqLog.logInformation("echo destDir:" + destDir);
        		m_xqLog.logInformation("echo destFile:" + destFile);
        		m_xqLog.logInformation("echo removeSrc:" + removeSrc);
        		m_xqLog.logInformation("echo replaceExisting:" + replaceExisting);
        		m_xqLog.logInformation("echo createDIr:" + createDir);
        		
                if (env != null) 
                {
                	try
                	{               		
                		java.io.File srcFileObject = new java.io.File(srcFile);
                		
            			if (!destDir.endsWith("/")) 
            			{
            				destDir = destDir + "/";
            			}
            			
            			if (destFile.equals("")	|| destFile.equals("\"\"")) 
            			{
            				destDir = destDir + srcFile.substring(srcFile.lastIndexOf("/") + 1);
            			} 
            			else 
            			{
            				destFile = destDir + destFile;
            			}
            			
            			java.io.File destFileObject = new java.io.File(destFile);
            			            			
            			//prevent overwrite of file
            			if (destFileObject.exists() && !replaceExisting) 
            			{
            				throw new XQServiceException("[FileCopyService] Runtime Error - Prevented Overwrite of file '" + destDir + "'.");
            			} 
            			else 
            			{	
            				java.io.BufferedInputStream inBuffer = new java.io.BufferedInputStream(
            						new java.io.FileInputStream(srcFileObject));	
            				
            				java.io.File destDirObject = new java.io.File(destDir);
            						
            				if (!destDirObject.exists()) 
            				{
            					if (createDir)
            					{
            					   (new java.io.File(destDir)).mkdirs();
            					}
            					else
            					{
            						throw new XQServiceException("[FileCopyService] Runtime Error - " +
            								"Destination Directory does not exist and the Create Directory " +
            								"flag for service set to FALSE.");
            					}
            				}
            				
            				java.io.BufferedOutputStream outBuffer = new java.io.BufferedOutputStream(
            						new java.io.FileOutputStream(destFileObject));
            				
            				byte[] buf_copy = new byte[1024];

            				int readLength;
            				
            				if (!srcFileObject.equals(destFileObject)) {
            					while ((readLength = inBuffer.read(
            							buf_copy, 0, 1024)) != -1)
            						outBuffer.write(buf_copy, 0,
            								readLength);
            				}

            				if (inBuffer != null) {
            					inBuffer.close();
            				}

            				if (outBuffer != null) {
            					outBuffer.close();
            				}
            				
            				if (removeSrc && !srcFileObject.equals(destFileObject)) {
            					//m_xqLog.logInformation("Running Delete of original file....");
            					srcFileObject.delete();
            				}
            			} // else
                	}
                	catch(Exception  e)
                	{
                		m_xqLog.logError("This service is unsupported, please goto PSDN for more information, and help at \"http://www.psdn.com/library/forum.jspa?forumID=97\"" + "\n" +
                				         "Error during attempt to copy file: '" + srcFile + "'.\n" +
                		                 e.getMessage());
                		                		
                		throw new XQServiceException(e.getMessage());
                	} 
                	

                    // Pass message onto the outbox.
                    Iterator addressList = env.getAddresses();
                    if (addressList.hasNext())  {
                        // Add the message to the Outbox
                        ctx.addOutgoing(env);
                    }
                }
            }
        }
    }
    
    /**
     * Clean up and get ready to destroy the service.
     *
     * <p> This method implement a required XQService method.
     */
    public void destroy() {
    }
}